@@ -47,6 +47,17 @@
 #define WACOM_ERASER_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.eraser"
 #define WACOM_BUTTON_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.tablet-button"
 
+static struct {
+	GnomeRRRotation  rotation;
+	GsdWacomRotation rotation_wacom;
+	const gchar     *rotation_string;
+} rotation_table[] = {
+	{ GNOME_RR_ROTATION_0,   GSD_WACOM_ROTATION_NONE, "none" },
+	{ GNOME_RR_ROTATION_90,  GSD_WACOM_ROTATION_CCW,  "ccw"  },
+	{ GNOME_RR_ROTATION_180, GSD_WACOM_ROTATION_HALF, "half" },
+	{ GNOME_RR_ROTATION_270, GSD_WACOM_ROTATION_CW,   "cw"   }
+};
+
 static WacomDeviceDatabase *db = NULL;
 
 struct GsdWacomStylusPrivate
@@ -558,7 +569,7 @@ find_builtin_output (GnomeRRScreen *rr_screen)
 			continue;
 
 		if (gnome_rr_output_is_laptop(rr_outputs[i])) {
-			retval = g_object_ref (rr_outputs[i]);
+			retval = rr_outputs[i];
 			break;
 		}
 	}
@@ -716,6 +727,18 @@ set_display_by_output (GsdWacomDevice  *device,
 	g_free (o_serial_s);
 }
 
+static GsdWacomRotation
+get_rotation_wacom (GnomeRRRotation rotation)
+{
+        guint i;
+
+        for (i = 0; i < G_N_ELEMENTS (rotation_table); i++) {
+                if (rotation_table[i].rotation & rotation)
+                        return (rotation_table[i].rotation_wacom);
+        }
+        g_assert_not_reached ();
+}
+
 void
 gsd_wacom_device_set_display (GsdWacomDevice *device,
                               int             monitor)
@@ -733,7 +756,7 @@ gsd_wacom_device_set_display (GsdWacomDevice *device,
 		return;
 	}
 
-	if (monitor >= 0)
+	if (monitor > GSD_WACOM_SET_ALL_MONITORS)
 		output = find_output_by_monitor (rr_screen, gdk_screen_get_default (), monitor);
 	set_display_by_output (device, output);
 
@@ -800,25 +823,25 @@ gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 	GnomeRRCrtc *crtc;
 	gint area[4];
 
-        g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), -1);
+        g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), GSD_WACOM_SET_ALL_MONITORS);
 
 	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
 	if (rr_screen == NULL) {
 		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
 		g_error_free (error);
-		return -1;
+		return GSD_WACOM_SET_ALL_MONITORS;
 	}
 
 	rr_output = find_output (rr_screen, device);
 	if (rr_output == NULL) {
 		g_object_unref (rr_screen);
-		return -1;
+		return GSD_WACOM_SET_ALL_MONITORS;
 	}
 
 	if (!is_on (rr_output)) {
 		g_warning ("Output is not active.");
 		g_object_unref (rr_screen);
-		return -1;
+		return GSD_WACOM_SET_ALL_MONITORS;
 	}
 
 	crtc = gnome_rr_output_get_crtc (rr_output);
@@ -832,7 +855,7 @@ gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 
 	if (area[2] <= 0 || area[3] <= 0) {
 		g_warning ("Output has non-positive area.");
-		return -1;
+		return GSD_WACOM_SET_ALL_MONITORS;
 	}
 
 	g_debug ("Area: %d,%d %dx%d", area[0], area[1], area[2], area[3]);
@@ -871,6 +894,32 @@ gsd_wacom_device_get_display_matrix (GsdWacomDevice *device, float matrix[NUM_EL
 	return TRUE;
 }
 
+GsdWacomRotation
+gsd_wacom_device_get_display_rotation (GsdWacomDevice *device)
+{
+	GError *error = NULL;
+	GnomeRRScreen *rr_screen;
+	GnomeRROutput *rr_output;
+	GnomeRRRotation rotation = GNOME_RR_ROTATION_0;
+
+	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
+	if (rr_screen == NULL) {
+		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+		g_error_free (error);
+		return GSD_WACOM_ROTATION_NONE;
+	}
+
+	rr_output = find_output (rr_screen, device);
+	if (rr_output) {
+		GnomeRRCrtc *crtc = gnome_rr_output_get_crtc (rr_output);
+		if (crtc)
+			rotation = gnome_rr_crtc_get_current_rotation (crtc);
+	}
+	g_object_unref (rr_screen);
+
+	return get_rotation_wacom (rotation);
+}
+
 static void
 add_stylus_to_device (GsdWacomDevice *device,
 		      const char     *settings_path,
@@ -1746,6 +1795,34 @@ gsd_wacom_device_get_button (GsdWacomDevice   *device,
 	}
 }
 
+GsdWacomRotation
+gsd_wacom_device_rotation_name_to_type (const char *rotation)
+{
+        guint i;
+
+	g_return_val_if_fail (rotation != NULL, GSD_WACOM_ROTATION_NONE);
+
+        for (i = 0; i < G_N_ELEMENTS (rotation_table); i++) {
+                if (strcmp (rotation_table[i].rotation_string, rotation) == 0)
+                        return (rotation_table[i].rotation_wacom);
+        }
+
+	return GSD_WACOM_ROTATION_NONE;
+}
+
+const char *
+gsd_wacom_device_rotation_type_to_name (GsdWacomRotation type)
+{
+        guint i;
+
+        for (i = 0; i < G_N_ELEMENTS (rotation_table); i++) {
+                if (rotation_table[i].rotation_wacom == type)
+                        return (rotation_table[i].rotation_string);
+        }
+
+	return "none";
+}
+
 GsdWacomDevice *
 gsd_wacom_device_create_fake (GsdWacomDeviceType  type,
 			      const char         *name,
